# -*- coding: utf-8 -*-


import os
import json
from .baseufile import BaseUFile
from .util import _check_dict, initialsharding_url, finishsharding_url, shardingupload_url, _file_iter, mimetype_from_file, mimetype_from_buffer
from .httprequest import ResponseInfo, _initialsharding, _finishsharding, _shardingupload
from .logger import logger
from .compact import s
from . import config
import time
import threading


class MultipartUploadUFile(BaseUFile):
    """
    分片上传UFile类

    Attributes:
    uploadid: string类型，初始化分片上传请求获得的uploadid
    etaglist: list 类型，已经成功上传分片的etag列表
    blocksize: integer类型，分片大小
    pausepartnumber: 第一个上传失败的分片编号（编号从0开始）
    """

    def __init__(self, public_key, private_key):
        """
        初始化 MultipartUploadUFile 实例

        :param public_key: string类型, 账户API公私钥中的公钥
        :param private_key: string类型, 账户API公私钥中的私钥
        :return: None，如果为非法的公私钥，则抛出ValueError异常
        """
        super(MultipartUploadUFile, self).__init__(public_key, private_key)
        self.uploadid = None
        self.blocksize = None
        self.etaglist = []
        self.__key = None
        self.__bucket = None
        self.__header = None
        self.__datastream = None
        self.__mimetype = None
        self.pausepartnumber = 0

    def uploadstream(self, bucket, key, stream, maxthread=4, retrycount=3, retryinterval=5, mime_type=None, header=None):
        """
        分片上传二进制数据流到UFile空间

        :param bucket: 空间名称
        :param key: 上传数据在空间中的名称
        :param stream: file-like 对象或者二进制数据流
        :param maxthread: 限制的最大线程数
        :param mime_type: 上传数据的MIME类型
        :param retrycount: integer 类型，分片重传次数
        :param retryinterval: integer 类型，同个分片失败重传间隔，单位秒
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        self.__bucket = bucket
        self.__key = key
        self.etaglist = []
        self.uploadid = None
        self.blocksize = None
        self.__header = header
        self.__stream = stream
        self.pausepartnumber = 0
        if self.__header is None:
            self.__header = dict()
        else:
            _check_dict(self.__header)
        if 'User-Agent' not in self.__header:
            self.__header['User-Agent'] = config.get_default('user_agent')

        # initial sharding request
        ret, resp = self.__initialsharding()
        if resp.ok():
            self.uploadid = ret.get('UploadId')
            self.blocksize = ret.get('BlkSize')
            logger.info('multipart upload id: {0}'.format(self.uploadid))
        else:
            logger.error('multipar upload init failed. error message: {0}'.format(resp.error))
            return ret, resp

        # mulitple sharding upload
        if mime_type is None:
            if hasattr(self.__stream, 'seek') and hasattr(self.__stream, 'read'):
                self.__mimetype = s(mimetype_from_buffer(self.__stream.read(1024)))
                self.__stream.seek(0, os.SEEK_SET)
            else:
                self.__mimetype = 'application/octec-stream'
        else:
            self.__mimetype = mime_type

        self.__header['Content-Type'] = self.__mimetype
        authorization = self.authorization('put', self.__bucket, self.__key, self.__header)
        self.__header['Authorization'] = authorization
        
        sem=threading.Semaphore(maxthread)  
        for data in _file_iter(self.__stream, self.blocksize):
            self.etaglist.append("")
            thread1 = threading.Thread(target=self.__partthread, args=(sem, self.__bucket, self.__key, self.uploadid, self.pausepartnumber, self.__header, data, retrycount, self.etaglist, ))
            thread1.start()
            self.pausepartnumber += 1
                            
        while threading.active_count()>1:
            time.sleep(0.05)
                
        logger.info('start finish sharding request.')
        ret, resp = self.__finishupload()
        if not resp.ok():
            logger.error('multipart upload failed. uploadid:{0}, pausepartnumber: {1}, key: {2} FAIL!!!'.format(self.uploadid, self.pausepartnumber, self.__key))
        else:
            logger.info('mulitpart upload succeed. uploadid: {0}, key: {1} SUCCEED'.format(self.uploadid, self.__key))
        return ret, resp

    def uploadfile(self, bucket, key, localfile, maxthread=4, retrycount=3, retryinterval=5, header=None):
        """
        分片上传本地文件到空间

        :param bucket: string类型，空间名称
        :param key: string类型，文件在空间中的名称
        :param localfile: string 类型，本地文件名称
        :param maxthread: 限制的最大线程数
        :param retrycount: integer 类型，分片重传次数
        :param retryinterval: integer 类型，同个分片失败重传间隔，单位秒
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        self.__localfile = localfile
        mime_type = s(mimetype_from_file(self.__localfile))
        with open(localfile, 'rb') as fd:
            return self.uploadstream(bucket, key, fd, maxthread, retrycount, retryinterval, mime_type, header)


    def __initialsharding(self):
        """
        初始化分片请求

        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if self.__header is None:
            self.__header = {}
        else:
            _check_dict(self.__header)
        if 'User-Agent' not in self.__header:
            self.__header['User-Agent'] = config.get_default('user_agent')

        self.__header['Content-Length'] = str(0)
        self.__header['Content-Type'] = 'text/plain'
        authorization = self.authorization('post', self.__bucket, self.__key, self.__header)
        self.__header['Authorization'] = authorization

        url = initialsharding_url(self.__bucket, self.__key)

        logger.info('start initialize sharding')
        logger.info('initial sharding url: {0}'.format(url))

        return _initialsharding(url, self.__header)

    def __finishupload(self):
        """
        完成分片上传的请求

        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if self.__header is None:
            self.__header = dict()
        else:
            _check_dict(self.__header)
        if 'User-Agent' not in self.__header:
            self.__header['User-Agent'] = config.get_default('user_agent')

        self.__header['Content-Type'] = 'text/plain'
        logger.info(self.etaglist)
        data = ','.join(self.etaglist)
        logger.info(data)
        self.__header['Content-Length'] = str(len(data))

        authorization = self.authorization('post', self.__bucket, self.__key, self.__header)
        self.__header['Authorization'] = authorization
        logger.info(json.dumps(self.__header, indent=4))
        params = {'uploadId': self.uploadid}

        url = finishsharding_url(self.__bucket, self.__key)

        logger.info('start finish sharding request')
        return _finishsharding(url, params, self.__header, data)

    def resumeuploadfile(self, retrycount=3, retryinterval=5, bucket=None, key=None, uploadid=None, blocksize=None, etaglist=None, localfile=None, pausepartnumber=None, mime_type=None, header=None):
        """
        断点续传失败的本地文件分片
        可以在调用uploadfile失败后重新续传，也可以通过传递所有需要的参数续传

        :param retrycount: integer 类型，分片重传次数
        :param retryinterval: integer 类型，同个分片失败重传间隔，单位秒
        :param bucket: string类型, 空间名称
        :param key: string类型，文件或数据在空间中的名称
        :param uploadid: string类型，初始化分片获得的uploadid
        :param blocksize: integer类型，分片大小
        :param etaglist: list类型，元素为已经上传成功的分片的etag
        :param pausepartnumber: integer类型，第一个失败分片的编号（编号从0开始）
        :param localfile: string类型，本地文件名称
        :param mime_type: string类型，上传数据的MIME类型
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if localfile:
            self.__localfile = localfile
        if blocksize:
            self.blocksize = blocksize
        if pausepartnumber:
            self.pausepartnumber = pausepartnumber
        if mime_type:
            self.__mimetype = mime_type
        else:
            mime_type = s(mimetype_from_file(self.__localfile))
        with open(self.__localfile, 'rb') as fd:
            fd.seek(self.pausepartnumber * self.blocksize, os.SEEK_SET)
            return self.resumeuploadstream(retrycount, retryinterval, bucket, key, uploadid, blocksize, etaglist, fd, pausepartnumber, mime_type, header)

    def resumeuploadstream(self, retrycount=3, retryinterval=5, bucket=None, key=None, uploadid=None, blocksize=None, etaglist=None, stream=None, pausepartnumber=None, mime_type=None, header=None):
        """
        断点续传失败数据流的分片
        可以在调用uploadstream失败后重新续传，也可以通过传递所有需要的参数续传

        :param retrycount: integer 类型，分片重传次数
        :param retryinterval: integer 类型，同个分片失败重传间隔，单位秒
        :param bucket: string类型, 空间名称
        :param key: string类型，文件或数据在空间中的名称
        :param uploadid: string类型，初始化分片获得的uploadid
        :param blocksize: integer类型，分片大小
        :param etaglist: list类型，元素为已经上传成功的分片的etag
        :param pausepartnumber: integer类型，第一个失败分片的编号（编号从0开始）
        :param stream: file-like对象或者二进制数据流，需要重新上传的数据
        :param mime_type: string类型，上传数据的MIME类型
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """

        if bucket:
            self.__bucket = bucket
        if key:
            self.__key = key
        if uploadid:
            self.uploadid = uploadid
        if blocksize:
            self.blocksize = blocksize
        if stream:
            self.__stream = stream
        if etaglist:
            self.etaglist = etaglist
        if pausepartnumber:
            self.pausepartnumber = pausepartnumber
        if header:
            self.__header = header
        if mime_type is not None:
            self.__mimetype = mime_type
        elif self.__mimetype is None:
            self.__mimetype = 'application/octec-stream'

        if self.__header is None:
            self.__header = dict()
        else:
            _check_dict(self.__header)
        if 'User-Agent' not in self.__header:
            self.__header['User-Agent'] = config.get_default('user_agent')

        # initial sharding request
        if self.uploadid is None:
            ret, resp = self.__initialsharding()
            if resp.ok():
                self.uploadid = ret.get('UploadId')
                self.blocksize = ret.get('BlkSize')
                logger.info('multipart upload id: {0}'.format(self.uploadid))
            else:
                logger.error('multipar upload init failed. error message: {0}'.format(resp.error))
                return ret, resp

        self.__header['Content-Type'] = self.__mimetype
        authorization = self.authorization('put', self.__bucket, self.__key, self.__header)
        self.__header['Authorization'] = authorization

        for data in _file_iter(self.__stream, self.blocksize):
            url = shardingupload_url(self.__bucket, self.__key, self.uploadid, self.pausepartnumber)
            ret = None
            resp = None
            for index in range(retrycount):
                logger.info('retry {0} time sharding upload sharing {0}'.format(index + 1, self.pausepartnumber))
                logger.info('sharding url:{0}'.format(url))
                ret, resp = _shardingupload(url, data, self.__header)
                if not resp.ok():
                    logger.error('failed {0} time when retry upload sharding {1},error message: {2}, uploadid: {3}'.format(index + 1, self.pausepartnumber, resp.error, self.uploadid))
                    if index < retrycount - 1:
                        time.sleep(retryinterval)
                else:
                    break
            if not resp.ok():
                logger.error('retry upload sharding {0} failed, uploadid: {1}'.format(self.pausepartnumber, self.uploadid))
                return ret, resp
            logger.info('retry upload sharding {0} succeed. etag: {1}, uploadid: {2}'.format(self.pausepartnumber, resp.etag, self.uploadid))
            self.pausepartnumber += 1
            self.etaglist.append(resp.etag)
        # finish sharding upload
        logger.info('start finish upload request')
        ret, resp = self.__finishupload()
        if not resp.ok():
            logger.error('multipart upload failed. uploadid:{0}, pausepartnumber: {1}, key: {2} FAIL!!!'.format(self.uploadid, self.pausepartnumber, self.__key))
        else:
            logger.info('mulitpart upload succeed. uploadid: {0}, key: {1} SUCCEED!!!'.format(self.uploadid, self.__key))
        return ret, resp

    def __partthread(self, sem, bucket, key, uploadid, part_number, header, data, retrycount, etaglist):
        with sem:
            url = shardingupload_url(bucket, key, uploadid, part_number)
            ret = None
            resp = None
            for index in range(retrycount):
                logger.info('try {0} time sharding upload sharding {1}'.format(index + 1, part_number))
                logger.info('sharding url:{0}'.format(url))
                ret, resp = _shardingupload(url, data, header)
                if not resp.ok():
                    logger.error('failed {0} time when upload sharding {1}.error message: {2}, uploadid: {3}'.format(index + 1, part_number, resp.error, upload_id))
                    if index < retrycount - 1:
                        time.sleep(retryinterval)
                else:
                    break
                
            if not resp.ok():
                logger.error('upload sharding {0} failed. uploadid: {1}'.format(part_number, uploadid))
                exit("part upload failed!")
            logger.info('upload sharding {0} succeed.etag:{1}, uploadid: {2}'.format(part_number, resp.etag, uploadid))
            etaglist[part_number] = resp.etag



# -*- coding: utf-8 -*-

from .compact import *
from . import config

import base64
import hashlib
import os
import struct

import mimetypes
from os import path

_EXTRA_TYPES_MAP = {
    ".js": "application/javascript",
    ".xlsx": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    ".xltx": "application/vnd.openxmlformats-officedocument.spreadsheetml.template",
    ".potx": "application/vnd.openxmlformats-officedocument.presentationml.template",
    ".ppsx": "application/vnd.openxmlformats-officedocument.presentationml.slideshow",
    ".pptx": "application/vnd.openxmlformats-officedocument.presentationml.presentation",
    ".sldx": "application/vnd.openxmlformats-officedocument.presentationml.slide",
    ".docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    ".dotx": "application/vnd.openxmlformats-officedocument.wordprocessingml.template",
    ".xlam": "application/vnd.ms-excel.addin.macroEnabled.12",
    ".xlsb": "application/vnd.ms-excel.sheet.binary.macroEnabled.12",
    ".apk": "application/vnd.android.package-archive",
    ".ipa": "application/vnd.ios.package-archive"
}


def urlsafe_b64encode(data):
    """
    urlsafe base64 encode

    :param data: string类型，待编码的字符串
    :return: string类型，编码的字符串
    """
    return s(base64.urlsafe_b64encode(b(data)))


def urlsafe_b64decode(data):
    """
    urlsafe base64 decode

    :param data: string类型，待解码的字符串
    :return: string类型，解码的字符串
    """
    return b(base64.urlsafe_b64decode(s(data)))


def standard_b64encode(data):
    """
    standard base64 encode

    :param data: string类型，待编码的字符串
    :return: string类型，编码的字符串
    """
    return s(base64.standard_b64encode(b(data)))


def standard_b64decode(data):
    """
    standard base64 decode

    :param data: string类型，待解码的字符串
    :return: string类型，解码的字符串
    """
    return b(base64.standard_b64decode(s(data)))


def _file_iter(input_stream, size):
    """
    二进制流迭代器

    :param input_stream: 二进制流
    :param size: integer类型，每次读取的块的大小
    :return: 指定大小的二进制块，如果读取失败会抛出IOError的异常
    """
    d = input_stream.read(size)
    while d:
        yield d
        d = input_stream.read(size)


def file_etag(localfile, size):
    """
    计算本地文件的etag

    :param localfile: string类型, 本地文件名
    :param size: integer类型, 分块大小
    :return: string类型, 本地文件的etag
    """

    filesize = os.path.getsize(localfile)
    blockcnt = filesize // size if filesize % size == 0 else filesize // size + 1

    hashstr = b''
    with open(localfile, 'rb') as input_stream:
        for block in _file_iter(input_stream, size):
            sha = hashlib.new('sha1')
            sha.update(b(block))
            hashstr += sha.digest()

    if blockcnt > 1:
        sha = hashlib.sha1()
        sha.update(hashstr)
        hashstr = sha.digest()

    return urlsafe_b64encode(struct.pack('@I', blockcnt) + hashstr)


def _check_dict(data):
    """
    check the type of data

    :param data: 键值对类型分别为string类型的dict类型
    :return: boolean类型，如果类型正确则返回True，否则抛出ValueError异常
    """
    if data is not None and isinstance(data, dict):
        return True
    raise ValueError('The input is not a dict-like object')


def ufile_put_url(bucket, key):
    """
    采用普通上传方法上传UCloud UFile文件的url

    :param bucket: string类型, 待创建的空间名称
    :param key:  string类型, 在空间中的文件名
    :return: string类型, 普通上传UFile的url
    """
    return 'http://{0}{1}/{2}'.format(bucket, config.get_default('upload_suffix'), key)


def ufile_post_url(bucket):
    """
    采用表单上传方法上传UCloud UFile文件的url

    :param bucket: string类型, 待创建的空间名称
    :return: string类型, 表单上传UFile的url
    """
    return 'http://{0}{1}/'.format(bucket, config.get_default('upload_suffix'))


def ufile_uploadhit_url(bucket):
    """
    秒传UCloud UFile文件的url

    :param bucket: string类型, 待创建的空间名称
    :return: string类型, 秒传UFile的url
    """
    return 'http://{0}{1}/uploadhit'.format(bucket, config.get_default('upload_suffix'))


def initialsharding_url(bucket, key):
    """
    初始化分片上传UCloud UFile的url

    :param bucket: string类型, 待创建的空间名称
    :param key:  string类型, 在空间中的文件名
    :return: string类型, 初始化分片上传UFile的url
    """
    return 'http://{0}{1}/{2}?uploads'.format(bucket, config.get_default('upload_suffix'), key)


def finishsharding_url(bucket, key):
    """
    结束分片上传UCloud UFile的url

    :param bucket: string类型, 待创建的空间名称
    :param key:  string类型, 在空间中的文件名
    :return: string类型, 结束分片上传UFile的url
    """
    return ufile_put_url(bucket, key)


def shardingupload_url(bucket, key, uploadid, part_number):
    """
    分片上传UCloud UFile的url

    :param bucket: string类型, 待创建的空间名称
    :param key:  string类型, 在空间中的文件名
    :param uploadid: string类型, 初始化分片上传获得的uploadid字符串
    :param part_number: integer类型, 分片上传的编号,从0开始
    :return: string类型, 结束分片上传UFile的url
    """
    return 'http://{0}{1}/{2}?uploadId={3}&partNumber={4}'.format(bucket, config.get_default('upload_suffix'), key, uploadid, s(str(part_number)))

def ufile_getfilelist_url(bucket):
    """
    获取文件列表的url

    :param bucket: string 类型，获取的空间名称
    :return: string类型，获取文件列表的url
    """
    return 'http://{0}{1}/?list'.format(bucket, config.get_default('upload_suffix'))

def mimetype_from_file(file):
    """
    获取文件的mimetype

    :param bucket: string 类型，获取的文件名称
    :return: string类型，获取文件的mimetype
    """
    ext = path.splitext(file)[1].lower()
    if ext in _EXTRA_TYPES_MAP:
        return _EXTRA_TYPES_MAP[ext]
    elif mimetypes.guess_type(file)[0] is not None:
        return mimetypes.guess_type(file)[0]

    return 'application/unknowntype'

def mimetype_from_buffer(stream):
    """
    获取流对象的mimetype

    :param bucket: string 类型，获取的流对象名称
    :return: string类型，获取流对象的mimetype
    """
    return 'application/octet-stream'

def ufile_restore_url(bucket, key):
    """
    解冻冷存文件的url

    :param bucket: string类型, 待创建的空间名称
    :param key:  string类型, 在空间中的文件名
    :return: string类型, 解冻文件的url
    """
    return 'http://{0}{1}/{2}?restore'.format(bucket, config.get_default('upload_suffix'), key)

def ufile_classswitch_url(bucket, key):
    """
    文件存储类型转换的url

    :param bucket: string类型, 待创建的空间名称
    :param key:  string类型, 在空间中的文件名
    :return: string类型, 类型转换的url
    """
    return 'http://{0}{1}/{2}'.format(bucket, config.get_default('upload_suffix'), key)

def ufile_copy_url(bucket, key):
    """
    拷贝文件的url

    :param bucket: string类型, 待创建的空间名称
    :param key:  string类型, 在空间中的目标文件名
    :return: string类型, 拷贝文件的url
    """
    return 'http://{0}{1}/{2}'.format(bucket, config.get_default('upload_suffix'), key)

def ufile_rename_url(bucket, key):
    """
    重命名文件的url

    :param bucket: string类型, 待创建的空间名称
    :param key:  string类型, 在空间中的源文件名
    :param newkey:  string类型, 在空间中的目标文件名
    :return: string类型, 重命名文件的url
    """
    return 'http://{0}{1}/{2}'.format(bucket, config.get_default('upload_suffix'), key)



# -*- coding: utf-8 -*-


import platform
from . import __version__

_sys_info = '{0}: {1}'.format(platform.system(), platform.machine())
_python_ver = platform.python_version()

USER_AGENT = 'UCloud UFile Python SDK {0} ({1} : Python/{2})'.format(__version__, _sys_info, _python_ver)
UCLOUD_PROXY_SUFFIX = '.cn-bj.ufileos.com'
UCLOUD_DOWNLOAD_SUFFIX = '.cn-bj.ufileos.com'
UCLOUD_API_URL = 'http://api.ucloud.cn'
BLOCKSIZE = 1024 * 1024 * 4

_config = {
    'connection_timeout': None,
    'expires': 300,
    'upload_suffix': UCLOUD_PROXY_SUFFIX,
    'download_suffix': UCLOUD_DOWNLOAD_SUFFIX,
    'user_agent': USER_AGENT,
    'md5': False,
}


def get_default(key):
    """
    返回默认配置
    """
    global _config
    return None if key not in _config else _config[key]

def set_default(connection_timeout=None, expires=None, user_agent=None, uploadsuffix=None, downloadsuffix=None, md5=None):
    """
    设置默认配置

    :param connection_timeout: integer类型，网络请求超时时间
    :param expires: integer类型，文件下载链接失效时间
    :user_agent: string类型
    :uploadsuffix: string类型，上传地址后缀
    :downloadsuffix: string类型，下载地址后缀
    :md5: 布尔类型，上传文件是否携带MD5
    """
    global _config
    if connection_timeout:
        _config['connection_timeout'] = connection_timeout
    if expires:
        _config['expires'] = expires
    if user_agent:
        _config['user_agent'] = user_agent
    if uploadsuffix and isinstance(uploadsuffix, str):
        _config['upload_suffix'] = uploadsuffix
    if downloadsuffix and isinstance(downloadsuffix, str):
        _config['download_suffix'] = downloadsuffix
    if md5 != None:
        _config['md5'] = md5



# -*- coding: utf-8 -*-


import os
import time
import hashlib
import json   
from .baseufile import BaseUFile
from .httprequest import _put_stream, _put_file, _post_file, ResponseInfo, _uploadhit_file, _download_file, _delete_file, _getfilelist,_head_file, _restore_file, _classswitch_file, _copy_file, _rename_file
from .util import _check_dict, ufile_put_url, ufile_post_url, file_etag, ufile_uploadhit_url, ufile_getfilelist_url, mimetype_from_file, ufile_restore_url, ufile_classswitch_url, ufile_copy_url, ufile_rename_url
from .logger import logger
from .compact import b, s, u, url_parse
from . import config
from .config import BLOCKSIZE
import string


class FileManager(BaseUFile):
    """
    UCloud UFile普通上传文件类
    """
    def __init__(self, public_key, private_key):
        """
        初始化 PutUFile 实例

        :param public_key: string类型, 账户API公私钥中的公钥
        :param private_key: string类型, 账户API公私钥中的私钥
        :return: None，如果为非法的公私钥，则抛出ValueError异常
        """
        super(FileManager, self).__init__(public_key, private_key)

    def putstream(self, bucket, key, stream, mime_type=None, header=None):
        """
        上传二进制流到空间

        :param bucket: string类型，上传空间名称
        :param key:  string 类型，上传文件在空间中的名称
        :param stream: 二进制数据流,从文件指针位置开始发送数据,在调用时需调用者自己调整文件指针位置
        :param mime_type: 二进制数据流的MIME类型
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        if mime_type is None:
            mime_type = 'application/octet-stream'
        header['Content-Type'] = mime_type
        if config.get_default('md5') == True:
            m = hashlib.md5()
            m.update(stream.getvalue())
            header['Content-MD5'] = m.hexdigest()
            stream.seek(0, os.SEEK_SET)
        authorization = self.authorization('put', bucket, key, header)
        header['Authorization'] = authorization
        url = ufile_put_url(bucket, key)
        logger.info('start put stream to bucket {0} as {1}'.format(bucket, key))
        logger.info('put UFile url: {0}'.format(url))
        logger.info('request header:\n{0}'.format(json.dumps(header, indent=4)))
        return _put_stream(url, header, stream)

    def putfile(self, bucket, key, localfile, header=None):
        """
        upload localfile to bucket as key

        :param bucket: string类型，上传空间名称
        :param key:  string 类型，上传文件在空间中的名称
        :param localfile: string类型，本地文件名称
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')
        mime_type = s(mimetype_from_file(localfile))
        file_size = os.path.getsize(localfile)
        header['Content-Type'] = mime_type
        if config.get_default('md5') == True:
            with open(localfile, 'rb') as data:
                m = hashlib.md5()
                m.update(data.read())
                header['Content-MD5'] = m.hexdigest()
                data.seek(0, os.SEEK_SET)
        authorization = self.authorization('put', bucket, key, header)
        header['Authorization'] = authorization
        if file_size!=0:
            header['Content-Length'] = str(file_size)
        url = ufile_put_url(bucket, key)
        logger.info('start put file {0} to bucket {1} as {2}'.format(localfile, bucket, key))
        logger.info('put UFile url: {0}'.format(url))
        logger.info('request header:\n{0}'.format(json.dumps(header, indent=4)))
        return _put_file(url, header, localfile)

    def postfile(self, bucket, key, localfile, header=None):
        """
        表单上传文件到UFile空间

        :param bucket: string类型，上传空间名称
        :param key:  string 类型，上传文件在空间中的名称
        :param localfile: string类型，本地文件名称
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header = dict()
        _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')
        mime_type = s(mimetype_from_file(localfile))

        # update the request header content-type
        boundary = self.__make_boundary()
        header['Content-Type'] = 'multipart/form-data; boundary={0}'.format(boundary)

        # form fields
        authorization = self.authorization('post', bucket, key, header, mime_type)
        fields = dict()
        fields['FileName'] = key
        fields['Authorization'] = authorization
        with open(localfile, 'rb') as stream:
            postdata = self.__make_postbody(boundary, fields, stream, mime_type, localfile)

        # update the request header content-length
        header['Content-Length'] = str(len(postdata))

        # post url
        url = ufile_post_url(bucket)

        # start post file
        logger.info('start post file {0} to bucket {1} as {2}'.format(localfile, bucket, key))
        logger.info('post url is {0}'.format(url))

        return _post_file(url, header, postdata)

    def __make_boundary(self):
        """
        生成post内容主体的限定字符串

        :return:: string类型
        """
        t = time.time()
        m = hashlib.md5()
        m.update(b(str(t)))
        return m.hexdigest()

    def __make_postbody(self, boundary, fields, stream, mime_type, localfile):
        """
        生成post请求内容主体

        :param boundary: string类型，post内容主体的限定字符串
        :param fields: ditc类型，键值对类型分别为string类型
        :param stream: 可读的file-like object(file object 或者BytesIO)
        :param mime_type: string类型，上传文件或数据的MIME类型
        :param localfile: string类型，上传文件或数据的本地名称
        :return: 二进制数据流
        """

        binarystream = b''
        for (key, value) in fields.items():
            binarystream += b('--{0}\r\n'.format(boundary))
            binarystream += b('Content-Disposition: form-data; name="{0}"\r\n'.format(key))
            binarystream += b('\r\n')
            binarystream += b('{0}\r\n'.format(value))

        binarystream += b('--{0}\r\n'.format(boundary))
        binarystream += b('Content-Disposition: form-data; name="file"; filename="{0}"\r\n'.format(localfile))
        binarystream += b('Content-Type: {0}\r\n'.format(mime_type))
        binarystream += b('\r\n')

        binarystream += stream.read()
        binarystream += b('\r\n')
        binarystream += b('--{0}\r\n'.format(boundary))

        return binarystream

    def uploadhit(self, bucket, key, localfile, header=None):
        """
        尝试秒传文件到UFile空间

        :param bucket: string类型，上传空间名称
        :param key:  string 类型，上传文件在空间中的名称
        :param localfile: string类型，本地文件名称
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """

        if header is None:
            header = dict()
        _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        filesize = os.path.getsize(localfile)
        fileetags = file_etag(localfile, BLOCKSIZE)
        mimetype = s(mimetype_from_file(localfile))

        # update request header
        header['Content-Type'] = mimetype
        header['Content-Length'] = str(0)
        authorization = self.authorization('post', bucket, key, header)
        header['Authorization'] = authorization

        # parameter

        params = {'Hash': fileetags,
                  'FileName': key,
                  'FileSize': filesize}

        url = ufile_uploadhit_url(bucket)

        logger.info('start upload hit localfile {0} as {1} in bucket {2}'.format(localfile, key, bucket))
        logger.info('request url: {0}'.format(url))

        return _uploadhit_file(url, header, params)

    def download_file(self, bucket, key, localfile, isprivate=True, expires=config.get_default('expires'), content_range=None, header=None):
        """
        下载UFile文件并且保存为本地文件

        :param bucket: string类型, UFile空间名称
        :param key: string类型， 下载文件在空间中的名称
        :param localfile: string类型，要保存的本地文件名称
        :param isprivate: boolean类型，如果为私有空间则为True
        :param expires: integer类型，私有文件链接有效时间
        :param content_range: tuple类型，元素为两个整型
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        if isinstance(content_range, tuple) and len(content_range) == 2:
            header['Range'] = 'bytes=' + '-'.join(map(lambda x: str(x), content_range))

        if not isprivate:
            url = self.public_download_url(bucket, key)
        else:
            url = self.private_download_url(bucket, key, expires, header, True)

        logger.info('get ufile url:{0}'.format(url))

        return _download_file(url, header, localfile)

    def public_download_url(self, bucket, key):
        """
        从公共空间下载文件的url

        :param bucket: string类型, 空间名称
        :param key: string类型，下载数据在空间中的名称
        :return: string类型，下载数据的url
        """
        return 'http://{0}{1}/{2}'.format(bucket, config.get_default('download_suffix'), key)

    def private_download_url(self, bucket, key, expires=config.get_default('expires'), header=None, internal=False):
        """
        从私有空间下载文件的url

        :param bucket: string类型, 空间名称
        :param key: string类型，下载数据在空间中的名称
        :param expires:  integer类型, 下载链接有效时间，单位为秒
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: string, 从私有空间下载文件和数据的url
        """
        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')
        if expires is not None:
            expires += int(time.time())
            header['Expires'] = s(str(expires))
        signature = self.signature(bucket, key, 'get', header)
        query = { 'UCloudPublicKey': self._public_key(),
                  'Expires': str(expires),
                  'Signature': signature }
        query_str = url_parse(query)
        if internal:
            return 'http://{0}{1}/{2}?{3}'.format(bucket, config.get_default('download_suffix'), key, query_str)
        else:
            return 'http://{0}{1}/{2}?UCloudPublicKey={3}&Expires={4}&Signature={5}'.format(bucket, config.get_default('download_suffix'), key, self._public_key(), str(expires), signature)

    def private_head_url(self, bucket, key, expires=config.get_default('expires'), header=None):
        """
        从私有空间下载文件的url

        :param bucket: string类型, 空间名称
        :param key: string类型，下载数据在空间中的名称
        :param expires:  integer类型, 下载链接有效时间，单位为秒
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: string, 从私有空间下载文件和数据的url
        """
        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        if expires is not None:
            expires += int(time.time())
            header['Expires'] = s(str(expires))
        signature = self.signature(bucket, key, 'head', header)
        return 'http://{0}{1}/{2}?UCloudPublicKey={3}&Expires={4}&Signature={5}'.format(bucket, config.get_default('download_suffix'), key, self._public_key(), str(expires), signature)

    def deletefile(self, bucket, key, header=None):
        """
        删除空间中文件方法

        :param bucket: string类型, 空间名称
        :param key:  string类型, 被删除文件在空间中的名称
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        authorization = self.authorization('delete', bucket, key, header)
        header['Authorization'] = authorization

        logger.info('start delete file {0} in bucket {1}'.format(key, bucket))
        url = ufile_put_url(bucket, key)

        return _delete_file(url, header)

    def getfilelist(self, bucket, prefix=None, marker=None, limit=None, header=None):
        """
        获取bucket下的文件列表

        :param bucket: string 类型，空间名称
        :param prefix: string 类型，文件前缀, 默认为空字符串
        :param marker: string 类型，文件列表起始位置, 默认为空字符串
        :param limit: integer 类型，文件列表数目, 默认为20
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        header['Content-Length'] = str(0)
        authorization = self.authorization('get', bucket, '', header)
        header['Authorization'] = authorization
        param = dict()
        if marker is not None and (isinstance(marker, str) or isinstance(marker, unicode)):
            param['marker'] = s(marker)
        if prefix is not None and (isinstance(prefix, str) or isinstance(prefix, unicode)):
            param['prefix'] = s(prefix)
        if limit is not None and isinstance(limit, int):
            param['limit'] = s(str(limit))
        info_message = ''.join(['start get file list from bucket {0}'.format(bucket), '' if marker is None else ', marker: {0}'.format(marker if isinstance(marker, str) else marker.encode('utf-8')), '' if limit is None else ', limit: {0}'.format(limit), '' if prefix is None else ', prefix: {0}'.format(prefix)])
        logger.info(info_message)
        url = ufile_getfilelist_url(bucket)
        return _getfilelist(url, header, param)

    def head_file(self,bucket,key,header=None):
        """
        获取空间中文件信息方法

        :param bucket: string类型, 空间名称
        :param key:  string类型, 文件在空间中的名称
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header=dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        authorization = self.authorization('head', bucket, key, header)
        header['Authorization'] = authorization

        logger.info('start head file {0} in bucket {1}'.format(key, bucket))
        url = ufile_put_url(bucket, key)

        return _head_file(url, header)

    def compare_file_etag(self,bucket,remotekey,localfile):
        """
        比对空间文件和本地文件方法

        :param bucket: string类型, 空间名称
        :param remotekey:  string类型, 文件在空间中的名称
        :param localfile: string类型，本地文件的路径
        :return:True为比对一致，False为不一致
        """
        ret,resp=self.head_file(bucket,remotekey)
        remote_etag=resp.etag.strip('\"')
        local_etag=file_etag(localfile,BLOCKSIZE)
        return (remote_etag==local_etag)

    def restore_file(self,bucket,key,header=None):
        """
        解冻冷存文件方法

        :param bucket: string类型, 空间名称
        :param key:  string类型, 文件在空间中的名称
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header=dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        authorization = self.authorization('put', bucket, key, header)
        header['Authorization'] = authorization

        logger.info('start restore file {0} in bucket {1}'.format(key, bucket))
        url = ufile_restore_url(bucket, key)

        return _restore_file(url, header)

    def class_switch_file(self,bucket,key,storageclass,header=None):
        """
        文件存储类型转换方法

        :param bucket: string类型, 空间名称
        :param key:  string类型, 文件在空间中的名称
        :param storageclass:  string类型, 文件目标存储类型
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header=dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        authorization = self.authorization('put', bucket, key, header)
        header['Authorization'] = authorization

        # parameter
        params = {'storageClass': storageclass}

        logger.info('start switch file {0} storage class in bucket {1}'.format(key, bucket))
        url = ufile_classswitch_url(bucket, key)

        return _classswitch_file(url, header, params)

    def copy(self, bucket, key, srcbucket, srckey, header=None):
        """
        尝试拷贝文件到UFile空间

        :param bucket: string类型，上传空间名称
        :param key:  string 类型，新文件在空间中的名称
        :param srcbucket: string类型，源文件所在空间名称
        :param srckey: string类型，源文件名称
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """

        if header is None:
            header = dict()
        _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        # update request header
        header['X-Ufile-Copy-Source'] = "/" + srcbucket + "/" + srckey
        header['Content-Length'] = str(0)
        authorization = self.authorization('put', bucket, key, header)
        header['Authorization'] = authorization

        url = ufile_copy_url(bucket, key)

        logger.info('start copy {0} in {1} to {2} in {3}'.format(srckey, srcbucket, key, bucket))
        logger.info('request url: {0}'.format(url))

        return _copy_file(url, header)

    def rename(self, bucket, key, newkey, force='true', header=None):
        """
        重命名文件方法

        :param bucket: string类型, 空间名称
        :param key:  string类型, 源文件在空间中的名称
        :param newkey:  string类型, 文件重命名后的新名称
        :param force:  string类型, 是否强行覆盖文件，值为'true'会覆盖，其他值则不会
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header=dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        authorization = self.authorization('put', bucket, key, header)
        header['Authorization'] = authorization

        # parameter
        params = {'newFileName': newkey,
                  'force': force}

        logger.info('start rename {0} in bucket {1}'.format(key, bucket))
        url = ufile_rename_url(bucket, key)

        return _rename_file(url, header, params)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
import re

try:
    import setuptools
    setup = setuptools.setup
except ImportError:
    setuptools = None
    from distutils.core import setup

def read(*names, **kwargs):
    return io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8")
    ).read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='ufile',
    version=find_version("ufile/__init__.py"),
    description='UCloud UFile Python SDK',
    author_email='leo.li@ucloud.cn',
    url="https://github.com/ucloud/ufile-sdk-python",
    platforms='any',
    packages=['ufile'],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    install_requires=['requests'],

)



# -*- coding: utf-8 -*-

from .logger import logger
from .auth import Auth
from .util import _check_dict
from . import config
from .config import UCLOUD_API_URL
from .httprequest import _bucket_request, ResponseInfo
from .compact import s

class BucketManager(object):
    """
    UCloud UFile 空间管理类
    """


    def __init__(self, public_key, private_key):
        """
        初始化 BucketManager 实例

        :param public_key: string类型， 账户API公私钥的公钥
        :param private_key: string类型，账户API公私钥的私钥
        :return: None，如果为非法的公私钥则抛出ValueError异常
        """
        self.__auth = Auth(public_key, private_key)


    def set_keys(self, public_key, private_key):
        """
        重新设置账户API公私钥

        :param public_key: string类型， 账户API公私钥的公钥
        :param private_key: string类型，账户API公私钥的私钥
        :return: None，如果为非法的公私钥则抛出ValueError异常
        """
        self.__auth.set_keys(public_key, private_key)


    def createbucket(self, bucket, region, buckettype='private', domainlist=None, header=None, projectid=None):
        """
        创建新的空间

        :param bucket: string类型，空间名称
        :param region: string类型，bucket 所在的地理区域
        |值|含义|
        |---|---|
        |cn-bj|北京|
        |hk|香港|
        |cn-gd|广州|
        |cn-sh2|上海二|
        |dn-jakarta|雅加达|
        |us-ca|洛杉矶|
        :param buckettype: string类型，'private' 或者 'public'
        :param domainlist: list 类型， 要绑定的域名列表
        :param projectid:  string类型，项目ID
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """

        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')
        param = dict()
        param['Action'] = 'CreateBucket'
        param['BucketName'] = bucket
        param['Region'] = region
        param['Type'] = buckettype
        if domainlist is None:
            domainlist = []
        for number, item in enumerate(domainlist):
            param['Domain.{0}'.format(number)] = item
        if projectid is not None:
            param['ProjectId'] = projectid
        signature = self.__auth.bucket_signature(param)
        param['Signature'] = signature
        logger.info('start create bucket {0}'.format(bucket))
        return _bucket_request(UCLOUD_API_URL, param, header)


    def describebucket(self, bucket=None, offset=0, limit=10, header=None, projectid=None):
        """
        获取空间的信息，如果不提供空间名称，则获取所有空间的信息

        :param bucketname: string类型, 空间名称
        :param offset: integer类型, 起始空间编码，当提供空间名时无效
        :param limit: integer类型，获取空间数量，当提供具体空间名时无效
        :param projectid:  string类型，项目ID
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """

        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        param = dict()
        param['Action'] = 'DescribeBucket'
        if bucket is not None:
            param['BucketName'] = bucket
        param['Offset'] = s(str(offset))
        param['Limit'] = s(str(limit))
        if projectid is not None:
            param['ProjectId'] = projectid

        signature = self.__auth.bucket_signature(param)
        param['Signature'] = signature
        logger.info('start request the bucket {0} details'.format(bucket))
        return _bucket_request(UCLOUD_API_URL, param, header)

    def updatebucket(self, bucket, buckettype, header=None, projectid=None):
        """
        更新空间的属性

        :param bucket: string类型，空间名称
        :param buckettype: string类型， 'private' 或者 'string'
        :param projectid:  string类型，项目ID
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        param = dict()
        param['Action'] = 'UpdateBucket'
        param['BucketName'] = bucket
        param['Type'] = buckettype
        if projectid is not None:
            param['ProjectId'] = projectid

        signature = self.__auth.bucket_signature(param)
        param['Signature'] = signature

        return _bucket_request(UCLOUD_API_URL, param, header)

    def deletebucket(self, bucket, header=None, projectid=None):
        """
        删除空间

        :param bucket: string类型，空间名称
        :param projectid:  string类型，项目ID
        :param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        param = dict()
        param['Action'] = 'DeleteBucket'
        param['BucketName'] = bucket
        if projectid is not None:
            param['ProjectId'] = projectid

        signature = self.__auth.bucket_signature(param)
        param['Signature'] = signature
        logger.info('start delete bucket {0}'.format(bucket))
        return _bucket_request(UCLOUD_API_URL, param, header)

    def getfilelist(self, bucket, offset=0, limit=20, header=None, projectid=None):
        """
        获取空间中文件列表（该接口已不再使用，获取文件列表请使用filemanager中的同名接口）

        :param bucket: string类型,空间名称
        :param offset: integer类型，文件列表偏移位置
        :param limit: integer类型，返回文件数量
        :param projectid:  string类型，项目ID
        :return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        :return:  ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """

        if header is None:
            header = dict()
        else:
            _check_dict(header)
        if 'User-Agent' not in header:
            header['User-Agent'] = config.get_default('user_agent')

        param = dict()
        param['Action'] = 'GetFileList'
        param['BucketName'] = bucket
        param['Offset'] = s(str(offset))
        param['Limit'] = s(str(limit))
        if projectid is not None:
            param['ProjectId'] = projectid
        signature = self.__auth.bucket_signature(param)
        param['Signature'] = signature
        logger.info('start request the file list of bucket {0}'.format(bucket))
        return _bucket_request(UCLOUD_API_URL, param, header)
